/**
 * Wireguard config schema definitions
 *
 * @since 1.0.0
 */
import * as CommandExecutor from "@effect/platform/CommandExecutor";
import * as PlatformError from "@effect/platform/Error";
import * as FileSystem from "@effect/platform/FileSystem";
import * as Path from "@effect/platform/Path";
import * as Socket from "@effect/platform/Socket";
import * as ParseResult from "@effect/schema/ParseResult";
import * as Schema from "@effect/schema/Schema";
import * as Array from "effect/Array";
import * as Cause from "effect/Cause";
import * as Effect from "effect/Effect";
import * as HashMap from "effect/HashMap";
import * as Request from "effect/Request";
import * as Resolver from "effect/RequestResolver";
import * as Scope from "effect/Scope";
import * as InternetSchemas from "./InternetSchemas.js";
import * as WireguardControl from "./WireguardControl.js";
import * as WireguardErrors from "./WireguardErrors.js";
import * as WireguardInterface from "./WireguardInterface.js";
import * as WireguardKey from "./WireguardKey.js";
import * as WireguardPeer from "./WireguardPeer.js";
declare const WireguardConfig_base: Schema.Class<WireguardConfig, {
    /** TODO: Document */
    Address: InternetSchemas.$CidrBlockFromString;
    /** TODO: Document */
    Dns: Schema.PropertySignature<"?:", InternetSchemas.IPv4 | InternetSchemas.IPv6 | undefined, never, "?:", string | undefined, never>;
    /**
     * The value for this is a decimal-string integer corresponding to the
     * listening port of the interface.
     */
    ListenPort: Schema.Union<[InternetSchemas.$Port, Schema.transformOrFail<Schema.$String, InternetSchemas.$Port, never>]>;
    /**
     * The value for this is a decimal-string integer corresponding to the
     * fwmark of the interface. The value may 0 in the case of a set operation,
     * in which case it indicates that the fwmark should be removed.
     */
    FirewallMark: Schema.PropertySignature<"?:", number | undefined, never, "?:", number | null | undefined, never>;
    /**
     * The value for this key should be a lowercase hex-encoded private key of
     * the interface. The value may be an all zero string in the case of a set
     * operation, in which case it indicates that the private key should be
     * removed.
     */
    PrivateKey: Schema.brand<Schema.Schema<string, string, never>, "WireguardKey">;
    /** List of peers to add. */
    Peers: Schema.PropertySignature<":", readonly WireguardPeer.WireguardPeer[], never, "?:", readonly {
        readonly PublicKey: string;
        readonly Endpoint?: `${string}:${number}` | `${string}:${number}:${number}` | {
            readonly ip: string;
            readonly port: number;
        } | {
            readonly ip: string;
            readonly natPort: number;
            readonly listenPort: number;
        } | `[${string}]:${number}` | `[${string}]:${number}:${number}` | {
            readonly ip: string;
            readonly port: number;
        } | {
            readonly ip: string;
            readonly natPort: number;
            readonly listenPort: number;
        } | {
            readonly host: string;
            readonly port: number;
        } | {
            readonly host: string;
            readonly natPort: number;
            readonly listenPort: number;
        } | null | undefined;
        readonly PersistentKeepalive?: number | null | undefined;
        readonly AllowedIPs?: ReadonlySet<`${string}/${number}`> | null | undefined;
        readonly PresharedKey?: string | null | undefined;
    }[] | null | undefined, never>;
}, {
    readonly Address: InternetSchemas.CidrBlock;
    readonly Dns?: InternetSchemas.IPv4 | InternetSchemas.IPv6 | undefined;
    readonly ListenPort: InternetSchemas.PortBrand;
    readonly FirewallMark?: number | undefined;
    readonly PrivateKey: string & import("effect/Brand").Brand<"WireguardKey">;
    readonly Peers: readonly WireguardPeer.WireguardPeer[];
}, {
    readonly Address: `${string}/${number}`;
    readonly ListenPort: string | number;
    readonly PrivateKey: string;
    readonly Dns?: string | undefined;
    readonly FirewallMark?: number | null | undefined;
    readonly Peers?: readonly {
        readonly PublicKey: string;
        readonly Endpoint?: `${string}:${number}` | `${string}:${number}:${number}` | {
            readonly ip: string;
            readonly port: number;
        } | {
            readonly ip: string;
            readonly natPort: number;
            readonly listenPort: number;
        } | `[${string}]:${number}` | `[${string}]:${number}:${number}` | {
            readonly ip: string;
            readonly port: number;
        } | {
            readonly ip: string;
            readonly natPort: number;
            readonly listenPort: number;
        } | {
            readonly host: string;
            readonly port: number;
        } | {
            readonly host: string;
            readonly natPort: number;
            readonly listenPort: number;
        } | null | undefined;
        readonly PersistentKeepalive?: number | null | undefined;
        readonly AllowedIPs?: ReadonlySet<`${string}/${number}`> | null | undefined;
        readonly PresharedKey?: string | null | undefined;
    }[] | null | undefined;
}, never, {
    readonly Address: InternetSchemas.CidrBlock;
    readonly Dns?: InternetSchemas.IPv4 | InternetSchemas.IPv6 | undefined;
    readonly ListenPort: InternetSchemas.PortBrand;
    readonly FirewallMark?: number | undefined;
    readonly PrivateKey: string & import("effect/Brand").Brand<"WireguardKey">;
    readonly Peers: readonly WireguardPeer.WireguardPeer[];
}, {}, {}>;
/**
 * A wireguard configuration.
 *
 * @since 1.0.0
 * @category Schemas
 */
export declare class WireguardConfig extends WireguardConfig_base {
    /**
     * Writes a wireguard interface configuration to an INI file.
     *
     * @since 1.0.0
     * @category Constructors
     * @param file - The path to the INI file.
     */
    writeToFile: {
        (file: string): Effect.Effect<void, ParseResult.ParseError | PlatformError.PlatformError, FileSystem.FileSystem | Path.Path>;
    };
    /**
     * Starts a wireguard tunnel that will continue to run and serve traffic
     * even after the nodejs process exits.
     *
     * @since 1.0.0
     * @category Wireguard
     */
    up: {
        (interfaceObject?: WireguardInterface.WireguardInterface | undefined): Effect.Effect<WireguardInterface.WireguardInterface, Socket.SocketError | ParseResult.ParseError | Cause.UnknownException | PlatformError.PlatformError | WireguardErrors.WireguardError, FileSystem.FileSystem | Path.Path | CommandExecutor.CommandExecutor | WireguardControl.WireguardControl>;
    };
    /**
     * Starts a wireguard tunnel that will be gracefully shutdown and stop
     * serving traffic once the scope is closed.
     *
     * @since 1.0.0
     * @category Wireguard
     */
    upScoped: {
        (interfaceObject?: WireguardInterface.WireguardInterface | undefined): Effect.Effect<WireguardInterface.WireguardInterface, Socket.SocketError | ParseResult.ParseError | Cause.UnknownException | PlatformError.PlatformError | WireguardErrors.WireguardError, FileSystem.FileSystem | Path.Path | CommandExecutor.CommandExecutor | WireguardControl.WireguardControl | Scope.Scope>;
    };
}
/**
 * @since 1.0.0
 * @category Api interface
 */
export interface $WireguardIniConfig extends Schema.Annotable<$WireguardIniConfig, string, Schema.Schema.Encoded<typeof WireguardConfig>, never> {
}
/**
 * A wireguard configuration encoded in the INI format.
 *
 * @since 1.0.0
 * @category Schema Transformations
 * @see {@link WireguardConfig}
 */
export declare const WireguardIniConfig: $WireguardIniConfig;
/**
 * Generates two wireguard configurations, each with the other as a single peer
 * and shares their keys appropriately.
 *
 * @since 1.0.0
 * @category Constructors
 * @param aliceData - The data for the first peer.
 * @param bobData - The data for the second peer.
 */
export declare const generateP2PConfigs: {
    <AliceData extends InternetSchemas.SetupDataEncoded, BobData extends InternetSchemas.SetupDataEncoded>(options: {
        aliceData: AliceData;
        bobData: BobData;
    }): Effect.Effect<readonly [aliceConfig: WireguardConfig, bobConfig: WireguardConfig], ParseResult.ParseError | WireguardErrors.WireguardError, never>;
    <AliceData extends InternetSchemas.EndpointEncoded, BobData extends InternetSchemas.EndpointEncoded>(options: {
        aliceData: AliceData;
        bobData: BobData;
        cidrBlock: InternetSchemas.CidrBlock;
        addressStartingIndex?: number | undefined;
    }): Effect.Effect<readonly [aliceConfig: WireguardConfig, bobConfig: WireguardConfig], ParseResult.ParseError | WireguardErrors.WireguardError, never>;
};
/**
 * Generates a collection of wireguard configurations for a star network with a
 * single central hub node and many peers all connected to it where the peers
 * all trust each other.
 *
 * @since 1.0.0
 * @category Constructors
 * @param hubData - The data for the hub node.
 * @param spokeData - The data for the spoke nodes.
 */
export declare const generateStarConfigs: {
    <Nodes extends readonly [
        InternetSchemas.SetupDataEncoded,
        InternetSchemas.SetupDataEncoded,
        ...Array<InternetSchemas.SetupDataEncoded>
    ]>(options: {
        nodes: Nodes;
    }): Effect.Effect<readonly [hubConfig: WireguardConfig, spokeConfigs: Array.NonEmptyReadonlyArray<WireguardConfig>], ParseResult.ParseError | WireguardErrors.WireguardError, never>;
    <Nodes extends readonly [
        InternetSchemas.EndpointEncoded,
        InternetSchemas.EndpointEncoded,
        ...Array<InternetSchemas.EndpointEncoded>
    ]>(options: {
        nodes: Nodes;
        cidrBlock: InternetSchemas.CidrBlock;
        addressStartingIndex?: number | undefined;
    }): Effect.Effect<readonly [hubConfig: WireguardConfig, spokeConfigs: Array.NonEmptyReadonlyArray<WireguardConfig>], ParseResult.ParseError | WireguardErrors.WireguardError, never>;
};
/**
 * Generates a collection of wireguard configurations for a hub and spoke
 * network with a single central hub node and many peers all connected to it
 * where none of the peers trust each other.
 *
 * @since 1.0.0
 * @category Constructors
 * @param hubData - The data for the hub node.
 * @param spokeData - The data for the spoke nodes.
 */
export declare const generateHubSpokeConfigs: {
    <HubData extends InternetSchemas.SetupDataEncoded, SpokeData extends Array.NonEmptyReadonlyArray<InternetSchemas.SetupDataEncoded>>(options: {
        hubData: HubData;
        spokeData: SpokeData;
    }): Effect.Effect<readonly [hubConfig: WireguardConfig, spokeConfigs: Array.NonEmptyReadonlyArray<WireguardConfig>], ParseResult.ParseError | WireguardErrors.WireguardError, never>;
    <HubData extends InternetSchemas.EndpointEncoded, SpokeData extends Array.NonEmptyReadonlyArray<InternetSchemas.EndpointEncoded>>(options: {
        hubData: HubData;
        spokeData: SpokeData;
        cidrBlock: InternetSchemas.CidrBlock;
        addressStartingIndex?: number | undefined;
    }): Effect.Effect<readonly [hubConfig: WireguardConfig, spokeConfigs: Array.NonEmptyReadonlyArray<WireguardConfig>], ParseResult.ParseError | WireguardErrors.WireguardError, never>;
};
/**
 * Generates a collection of wireguard configurations.
 *
 * @since 1.0.0
 * @category Constructors
 */
export declare const generate: {
    <HubData extends InternetSchemas.SetupDataEncoded, SpokeData extends Array.NonEmptyReadonlyArray<InternetSchemas.SetupDataEncoded>>(options: {
        hubData: HubData;
        spokeData: SpokeData;
        preshareKeys?: HashMap.HashMap<SpokeData[number] | HubData, WireguardKey.WireguardKey> | "generate" | undefined;
        trustMap?: HashMap.HashMap<SpokeData[number], Array.NonEmptyReadonlyArray<SpokeData[number]>> | "trustAllPeers" | "trustNoPeers" | undefined;
    }): Effect.Effect<readonly [hubConfig: WireguardConfig, spokeConfigs: Array.NonEmptyReadonlyArray<WireguardConfig>], ParseResult.ParseError | WireguardErrors.WireguardError, never>;
    <HubData extends InternetSchemas.EndpointEncoded, SpokeData extends Array.NonEmptyReadonlyArray<InternetSchemas.EndpointEncoded>>(options: {
        hubData: HubData;
        spokeData: SpokeData;
        cidrBlock: InternetSchemas.CidrBlock;
        addressStartingIndex?: number | undefined;
        preshareKeys?: HashMap.HashMap<SpokeData[number] | HubData, WireguardKey.WireguardKey> | "generate" | undefined;
        trustMap?: HashMap.HashMap<SpokeData[number], Array.NonEmptyReadonlyArray<SpokeData[number]>> | "trustAllPeers" | "trustNoPeers" | undefined;
    }): Effect.Effect<readonly [hubConfig: WireguardConfig, spokeConfigs: Array.NonEmptyReadonlyArray<WireguardConfig>], ParseResult.ParseError | WireguardErrors.WireguardError, never>;
};
/**
 * Loads a wireguard interface configuration from an INI file.
 *
 * @since 1.0.0
 * @category Constructors
 * @param file - The path to the INI file.
 */
export declare const fromConfigFile: {
    (file: string): Effect.Effect<WireguardConfig, ParseResult.ParseError | PlatformError.PlatformError, FileSystem.FileSystem>;
};
/**
 * @since 1.0.0
 * @category Responses
 * @see https://www.wireguard.com/xplatform/
 */
export declare const WireguardGetConfigResponse: Schema.Schema<{
    readonly Address: InternetSchemas.CidrBlock;
    readonly Dns?: InternetSchemas.IPv4 | InternetSchemas.IPv6 | undefined;
    readonly ListenPort: InternetSchemas.PortBrand;
    readonly FirewallMark?: number | undefined;
    readonly PrivateKey: string & import("effect/Brand").Brand<"WireguardKey">;
    writeToFile: {
        (file: string): Effect.Effect<void, ParseResult.ParseError | PlatformError.PlatformError, FileSystem.FileSystem | Path.Path>;
    };
    up: {
        (interfaceObject?: WireguardInterface.WireguardInterface | undefined): Effect.Effect<WireguardInterface.WireguardInterface, Socket.SocketError | ParseResult.ParseError | Cause.UnknownException | PlatformError.PlatformError | WireguardErrors.WireguardError, FileSystem.FileSystem | Path.Path | CommandExecutor.CommandExecutor | WireguardControl.WireguardControl>;
    };
    upScoped: {
        (interfaceObject?: WireguardInterface.WireguardInterface | undefined): Effect.Effect<WireguardInterface.WireguardInterface, Socket.SocketError | ParseResult.ParseError | Cause.UnknownException | PlatformError.PlatformError | WireguardErrors.WireguardError, FileSystem.FileSystem | Path.Path | CommandExecutor.CommandExecutor | WireguardControl.WireguardControl | Scope.Scope>;
    };
    readonly Peers: readonly WireguardPeer.WireguardUApiGetPeerResponse[];
}, {
    readonly Address: `${string}/${number}`;
    readonly Dns?: string | undefined;
    readonly ListenPort: string | number;
    readonly FirewallMark?: number | null | undefined;
    readonly PrivateKey: string;
    readonly Peers?: readonly {
        readonly PublicKey: string;
        readonly Endpoint?: `${string}:${number}` | `${string}:${number}:${number}` | {
            readonly ip: string;
            readonly port: number;
        } | {
            readonly ip: string;
            readonly natPort: number;
            readonly listenPort: number;
        } | `[${string}]:${number}` | `[${string}]:${number}:${number}` | {
            readonly ip: string;
            readonly port: number;
        } | {
            readonly ip: string;
            readonly natPort: number;
            readonly listenPort: number;
        } | {
            readonly host: string;
            readonly port: number;
        } | {
            readonly host: string;
            readonly natPort: number;
            readonly listenPort: number;
        } | null | undefined;
        readonly PersistentKeepalive?: number | null | undefined;
        readonly AllowedIPs?: ReadonlySet<`${string}/${number}`> | null | undefined;
        readonly PresharedKey?: string | null | undefined;
        readonly rxBytes: string;
        readonly txBytes: string;
        readonly lastHandshakeTimeSeconds: string;
    }[] | null | undefined;
}, never>;
/**
 * @since 1.0.0
 * @category Responses
 */
export type WireguardGetConfigResponse = Schema.Schema.Type<typeof WireguardGetConfigResponse>;
declare const WireguardGetConfigRequest_base: new <Success, Error, A extends Record<string, any>>(args: import("effect/Types").Equals<Omit<A, typeof Request.RequestTypeId>, {}> extends true ? void : { readonly [P in keyof A as P extends "_tag" | typeof Request.RequestTypeId ? never : P]: A[P]; }) => Request.Request<Success, Error> & Readonly<A> & {
    readonly _tag: "WireguardGetConfigRequest";
};
/**
 * @since 1.0.0
 * @category Requests
 */
export declare class WireguardGetConfigRequest extends WireguardGetConfigRequest_base<Schema.Schema.Type<typeof WireguardGetConfigResponse>, ParseResult.ParseError | Socket.SocketError, {
    readonly address: InternetSchemas.CidrBlockFromStringEncoded;
    readonly wireguardInterface: WireguardInterface.WireguardInterface;
}> {
}
declare const WireguardSetConfigRequest_base: new <Success, Error, A extends Record<string, any>>(args: import("effect/Types").Equals<Omit<A, typeof Request.RequestTypeId>, {}> extends true ? void : { readonly [P in keyof A as P extends "_tag" | typeof Request.RequestTypeId ? never : P]: A[P]; }) => Request.Request<Success, Error> & Readonly<A> & {
    readonly _tag: "WireguardSetConfigRequest";
};
/**
 * @since 1.0.0
 * @category Requests
 */
export declare class WireguardSetConfigRequest extends WireguardSetConfigRequest_base<WireguardInterface.WireguardInterface, ParseResult.ParseError | Socket.SocketError, {
    readonly config: WireguardConfig;
    readonly wireguardInterface: WireguardInterface.WireguardInterface;
    readonly peerRequestMapper?: (peer: WireguardPeer.WireguardPeer) => {
        readonly remove?: boolean | undefined;
        readonly updateOnly?: boolean | undefined;
        readonly replaceAllowedIps?: boolean | undefined;
    } | undefined;
}> {
}
/**
 * @since 1.0.0
 * @category Resolvers
 */
export declare const WireguardGetConfigResolver: Resolver.RequestResolver<WireguardGetConfigRequest, never>;
/**
 * @since 1.0.0
 * @category Resolvers
 */
export declare const WireguardSetConfigResolver: Resolver.RequestResolver<WireguardSetConfigRequest, never>;
export {};
//# sourceMappingURL=WireguardConfig.d.ts.map